
/**
*
*/
package org.n52.sos.web;

import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;

import org.n52.sos.web.auth.CustomPrincipal;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

import org.n52.sos.binding.Binding;
import org.n52.sos.binding.BindingRepository;
import org.n52.sos.coding.OperationKey;
import org.n52.sos.exception.HTTPException;
import org.n52.sos.request.operator.RequestOperatorKey;
import org.n52.sos.request.operator.RequestOperatorRepository;
import org.n52.sos.service.Configurator;
import org.n52.sos.util.http.HTTPMethods;
import org.n52.sos.util.http.MediaType;

import com.google.common.base.Objects;

/**
* @since 4.0.0
*
*/
@Controller
@RequestMapping(ControllerConstants.Paths.CLIENT)
public class ClientController extends AbstractController {
   public static final String BINDINGS = "bindings";

   public static final String VERSIONS = "versions";

   public static final String OPERATIONS = "operations";

   @RequestMapping(method = RequestMethod.GET)
   public ModelAndView get() {

       Authentication auth = SecurityContextHolder.getContext().getAuthentication();

       if (Configurator.getInstance() != null) {
           return new ModelAndView(ControllerConstants.Views.CLIENT, OPERATIONS, getAvailableOperations()).addObject("apikey", ((CustomPrincipal)auth.getPrincipal()).getApikey());
       } else {
           return new ModelAndView(ControllerConstants.Views.CLIENT).addObject("apikey", ((CustomPrincipal)auth.getPrincipal()).getApikey());
       }
   }

   private List<AvailableOperation> getAvailableOperations() {
       final List<AvailableOperation> ops = new LinkedList<AvailableOperation>();
       for (RequestOperatorKey rokt : RequestOperatorRepository.getInstance().getActiveRequestOperatorKeys()) {
           final String service = rokt.getServiceOperatorKey().getService();
           final String version = rokt.getServiceOperatorKey().getVersion();
           final String operation = rokt.getOperationName();
           final OperationKey ok = new OperationKey(service, version, operation);
           for (Entry<String, Binding> b : BindingRepository.getInstance().getBindings().entrySet()) {
               try {
                   final String pattern = b.getKey();
                   final Binding binding = b.getValue();
                   if (binding.checkOperationHttpDeleteSupported(ok)) {
                       for (MediaType contentType : binding.getSupportedEncodings()) {
                           ops.add(new AvailableOperation(
                                   service, version, operation,
                                   contentType.toString(), HTTPMethods.DELETE));
                       }
                   }
                   if (binding.checkOperationHttpGetSupported(ok)) {
                       for (MediaType contentType : binding.getSupportedEncodings()) {
                           ops.add(new AvailableOperation(
                                   service, version, operation,
                                   contentType.toString(), HTTPMethods.GET));
                       }
                   }
                   if (binding.checkOperationHttpOptionsSupported(ok)) {
                       for (MediaType contentType : binding.getSupportedEncodings()) {
                           ops.add(new AvailableOperation(
                                   service, version, operation,
                                   contentType.toString(), HTTPMethods.OPTIONS));
                       }
                   }
                   if (binding.checkOperationHttpPostSupported(ok)) {
                       for (MediaType contentType : binding.getSupportedEncodings()) {
                           ops.add(new AvailableOperation(
                                   service, version, operation,
                                   contentType.toString(), HTTPMethods.POST));
                       }
                   }
                   if (binding.checkOperationHttpPutSupported(ok)) {
                       for (MediaType contentType : binding.getSupportedEncodings()) {
                           ops.add(new AvailableOperation(
                                   service, version, operation,
                                   contentType.toString(), HTTPMethods.PUT));
                       }
                   }
               } catch (HTTPException ex) {
                   /* ignore */
               }
           }
       }
       return ops;
   }

   public static class AvailableOperation {
       private final String service;

       private final String version;

       private final String operation;

       private final String contentType;

       private final String method;

       public AvailableOperation(String service, String version, String operation, String contentType, String method) {
           this.service = service;
           this.version = version;
           this.operation = operation;
           this.contentType = contentType;
           this.method = method;
       }

       public String getService() {
           return service;
       }

       public String getVersion() {
           return version;
       }

       public String getOperation() {
           return operation;
       }

       public String getContentType() {
           return contentType;
       }

       public String getMethod() {
           return method;
       }

       @Override
       public String toString() {
           return String.format("AvailableOperation[method=%s, service=%s, version=%s, operation=%s, contentType=%s]",
                   getMethod(), getService(), getVersion(), getOperation(), getContentType());
       }

       @Override
       public int hashCode() {
           return Objects.hashCode(getMethod(), getService(), getVersion(), getOperation(), getContentType());
       }

       @Override
       public boolean equals(Object obj) {
           if (obj instanceof AvailableOperation) {
               AvailableOperation other = (AvailableOperation) obj;
               return Objects.equal(getMethod(), other.getMethod())
                       && Objects.equal(getService(), other.getService())
                       && Objects.equal(getVersion(), other.getVersion())
                       && Objects.equal(getOperation(), other.getOperation())
                       && Objects.equal(getContentType(), other.getContentType());
           }
           return false;
       }
   }
}
