package org.n52.sos.web;

import es.xunta.amtega.transversal.authentication.AuthenticationManager;
import es.xunta.amtega.transversal.authentication.exception.CommunicationException;
import es.xunta.amtega.transversal.authentication.exception.InvalidLoginCredentialsException;
import org.apache.commons.io.IOUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.n52.sos.web.auth.CustomPrincipal;
import org.n52.sos.web.authorization.AuthorizationService;
import org.n52.sos.web.model.dto.*;
import org.n52.sos.web.model.exceptions.UsernameInUseException;
import org.n52.sos.web.model.helpers.Constants;
import org.n52.sos.web.model.services.ApiKeyService;
import org.n52.sos.web.model.services.RestriccionService;
import org.n52.sos.web.model.services.UserService;
import org.n52.sos.web.model.vo.ApiKeyVO;
import org.n52.sos.web.model.vo.UserVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;
import org.springframework.web.servlet.view.RedirectView;
import sun.misc.Request;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.invoke.MethodHandles;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

@Controller
public class ClienteController extends AbstractController {

    private static final Logger LOGGER = LogManager.getLogger(MethodHandles.lookup().lookupClass());

    @Autowired
    private AuthorizationService authorizationService;
    @Autowired
    private UserService userService;
    @Autowired
    private ApiKeyService apiKeyService;
    @Autowired
    private RestriccionService restriccionService;

    @RequestMapping(value = "cliente/login", method = RequestMethod.GET)
    public String login() {
        return "login";
    }

    @RequestMapping(value = "cliente/login/entrar", method = RequestMethod.POST)
    public String doLogin(@RequestParam("username") String username, @RequestParam("password") String password, Model model) {

        if (!username.matches("^([a-zA-Z0-9@\\\\.]{8,})")) {
            model.addAttribute("errorMessage","login.message.error.baddata");
            return "login";
        }

        if (!password.matches("^([a-zA-Z0-9@\\\\.]{8,})")) {
            model.addAttribute("errorMessage","login.message.error.baddata");
            return "login";
        }

        // Check if user exists
        UserVO userVO = userService.checkUser(username, password);

        if (userVO == null || userVO.getLDAP()) {
            model.addAttribute("errorMessage", "login.message.error.user.notfound");
            return "login";
        }

        if (!userVO.getEnabled()) {
            model.addAttribute("errorMessage", "login.message.error.user.diabled");
            return "login";
        }

        Integer userId = userVO.getId();
        String apiKey = null;
        try {
             apiKey = apiKeyService.findApiKeyByUserId(userId).getValue();
        } catch (SQLException e) {
            LOGGER.catching(e);
        }

        SecurityContextHolder.getContext().setAuthentication(
                new UsernamePasswordAuthenticationToken(
                        new CustomPrincipal(userId, apiKey),
                        SecurityContextHolder.getContext().getAuthentication().getCredentials(),
                        authorizationService.getAuthorities(userVO.getId()))
        );

        return "index";
    }

    @RequestMapping(value = "cliente/ldaplogin/entrar", method = RequestMethod.POST)
    public RedirectView doLDAPLogin(@RequestParam("username") String username, @RequestParam("password") String password, RedirectAttributes redirectAttributes) {

        if (!username.matches("^([a-zA-Z0-9@\\\\.]{8,})")) {
            redirectAttributes.addAttribute("errorMessage","login.message.error.baddata");
            return new RedirectView(getContext().getContextPath() + "/cliente/login");
        }

        if (!password.matches("^([a-zA-Z0-9@\\\\.]{8,})")) {
            redirectAttributes.addAttribute("errorMessage","login.message.error.baddata");
            return new RedirectView(getContext().getContextPath() + "/cliente/login");
        }

        Properties props = new Properties();
        props.setProperty("ldap.url", "ldap://xuntaappsad.xunta.local:389/DC=xunta,DC=local");
        AuthenticationManager auth = new AuthenticationManager(props);

        try {
            auth.login(username, password);
        } catch (InvalidLoginCredentialsException e) {
            LOGGER.catching(e);
            redirectAttributes.addAttribute("errorMessage", "login.message.error.user.notfound");
            return new RedirectView(getContext().getContextPath() + "/cliente/login");
        } catch (CommunicationException e) {
            LOGGER.catching(e);
            redirectAttributes.addAttribute("errorMessage", "login.message.error.communication");
            return new RedirectView(getContext().getContextPath() + "/cliente/login");
        }

        // Check if user exists, otherwise register it
        UserVO userVO = userService.checkUser(username, password);
        if (userVO == null) {
            // User logged in with LDAP but never logged before, so register it
            try {
                UserVO user = userService.register(username, password, "", "", "", true);

                String apiKey = apiKeyService.findApiKeyByUserId(user.getId()).getValue();

                SecurityContextHolder.getContext().setAuthentication(
                        new UsernamePasswordAuthenticationToken(
                                new CustomPrincipal(user.getId(), apiKey),
                                SecurityContextHolder.getContext().getAuthentication().getCredentials(),
                                authorizationService.getAuthorities(user.getId()))
                );
            } catch (SQLException e) {
                LOGGER.catching(e);
                redirectAttributes.addAttribute("errorMessage", "login.message.error.internalproblem");
                return new RedirectView(getContext().getContextPath() + "/cliente/login");
            } catch (Exception e) {
                LOGGER.catching(e);
                redirectAttributes.addAttribute("errorMessage", "login.message.error.internalproblem");
                return new RedirectView(getContext().getContextPath() + "/cliente/login");
            }
        } else {
            // User logged in with LDAP and had done it before
            Integer userId = userVO.getId();
            String apiKey = null;
            try {
                apiKey = apiKeyService.findApiKeyByUserId(userId).getValue();
            } catch (SQLException e) {
                LOGGER.catching(e);
                redirectAttributes.addAttribute("errorMessage", "login.message.error.internalproblem");
                return new RedirectView(getContext().getContextPath() + "/cliente/login");
            }

            SecurityContextHolder.getContext().setAuthentication(
                    new UsernamePasswordAuthenticationToken(
                            new CustomPrincipal(userId, apiKey),
                            SecurityContextHolder.getContext().getAuthentication().getCredentials(),
                            authorizationService.getAuthorities(userVO.getId()))
            );
        }

        return new RedirectView(getContext().getContextPath() + "/index");
    }

    @RequestMapping(value = "cliente/register", method = RequestMethod.GET)
    public String register() {
        return "register";
    }


    @RequestMapping(value = "cliente/register/registrarse", method = RequestMethod.POST)
    public String doRegister(
            @RequestParam(value = "username", required = true) String username,
            @RequestParam(value = "password", required = true) String password,
            @RequestParam(value = "repeatpassword", required = true) String repeatPassword,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "surname", required = false) String surname,
            @RequestParam(value = "organization", required = false) String organization,
            Model model
    ) {

        if (password==null || password.isEmpty() || repeatPassword==null || repeatPassword.isEmpty() || !password.equals(repeatPassword)) {
            model.addAttribute("errorMessage","register.message.error.passwordnotcorrect");
            return "register";
        }

        if (!username.matches("^([a-zA-Z0-9@\\\\.]{8,})")) {
            model.addAttribute("errorMessage","register.message.error.baddata");
            return "register";
        }
        if (!password.matches("^([a-zA-Z0-9@\\\\.]{8,})")) {
            model.addAttribute("errorMessage","register.message.error.baddata");
            return "register";
        }

        try {
            UserVO user = userService.register(username, password, name, surname, organization, false);

            String apiKey = apiKeyService.findApiKeyByUserId(user.getId()).getValue();

            SecurityContextHolder.getContext().setAuthentication(
                    new UsernamePasswordAuthenticationToken(
                            new CustomPrincipal(user.getId(), apiKey),
                            SecurityContextHolder.getContext().getAuthentication().getCredentials(),
                            authorizationService.getAuthorities(user.getId()))
            );

        } catch (UsernameInUseException e) {
            LOGGER.catching(e);
            model.addAttribute("errorMessage","register.message.error.usernamealreadyinuse");
            return "register";
        } catch (SQLException e) {
            LOGGER.catching(e);
            model.addAttribute("errorMessage","register.message.error.internalservererror");
            return "register";
        } catch (Exception e) {
            return "register";
        }

        return "index";
    }

    @RequestMapping(value = "cliente/controlpanel", method = RequestMethod.GET)
    public String panelControl(Model model) {

        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        Integer userId = ((CustomPrincipal)auth.getPrincipal()).getUserId();

        // Retrieve user's apikey from database
        try {
            ApiKeyVO apiKeyVO = apiKeyService.findApiKeyByUserId(userId);
            if (apiKeyVO==null || apiKeyVO.getValue().isEmpty()) {
                return "index";
            }
            model.addAttribute("apikey", apiKeyVO.getValue());
        } catch (SQLException e) {
            LOGGER.catching(e);
            return "index";
        }

        return "controlpanel";
    }

    @RequestMapping(value = "cliente/userscontrolpanel", method = RequestMethod.GET)
    public String usersManagementPanel(Model model) {

        List<UserDTO> users = userService.findAll();
        model.addAttribute("users", users);

        return "usersmanagmentpanel";
    }

    @RequestMapping(value = "cliente/users/status", method = RequestMethod.POST)
    public ResponseEntity<UserVO> usersManagementChangeUserStatus(@RequestParam("id") Integer id, @RequestParam("status") Boolean status) {
        UserVO userUpdated = userService.modifyStatus(id, status);
        return new ResponseEntity<>(userUpdated, HttpStatus.OK);
    }

    @RequestMapping(value = "cliente/legalnote", method = RequestMethod.GET)
    public String legalNote() {
        return "legalnote";
    }

    @RequestMapping(value = "cliente/sitemap", method = RequestMethod.GET)
    public String siteMap() {
        return "sitemap";
    }

    @RequestMapping(value = "cliente/capabilities/download/{format}", method = RequestMethod.GET)
    public ResponseEntity<byte[]> downloadCapabilitiesXML(@PathVariable String format) {
        File f = null;
        if (format.equals("xml")) {
            f = new File(System.getProperty("apps_data_path") + "/metsos/getCapabilities.txt");
        } else if (format.equals("json")) {
            f = new File(System.getProperty("apps_data_path") + "/metsos/getCapabilities.json");
        }
        try (InputStream in = new FileInputStream(f)) {
            HttpHeaders headers = new HttpHeaders();
            return new ResponseEntity<byte[]>(IOUtils.toByteArray(in), headers, HttpStatus.OK);
        } catch (IOException e) {
            LOGGER.catching(e);
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @RequestMapping(value = "admin/timerestrictionspanel", method=RequestMethod.GET)
    public String timeRestrictionsPanel(Model model) {

        List<RestriccionDTO> restricciones = restriccionService.recuperarTodasLasRestricciones();
        model.addAttribute("restrictions", restricciones);

        return "timerestrictionspanel";
    }

    @RequestMapping(value = "admin/timerestrictionspanel/delete/{id}", method = RequestMethod.GET)
    public String timeRestrictionsPanelDelete(@PathVariable Integer id) {
        // Delete restriction by Id
        Boolean result = restriccionService.borrarRestriccion(id);
        // Redirect to admin/timerestrictionspanel
        return "redirect:/admin/timerestrictionspanel";
    }

    @RequestMapping(value = "admin/timerestrictionspanel/new", method = RequestMethod.GET)
    public String timeRestrictionsPanelNew(Model model) {

        List<RestriccionTipoDTO> tipos = restriccionService.recuperarTodosLosTiposDeRestriccion();
        model.addAttribute("tipos", tipos);

        // TODO Crear e usar intervalosService para consultar os intervalos da base de datos MeteoDatos
        List<IntervaloMeteoDatosDTO> intervalosMeteoDatos = new ArrayList<>();
        intervalosMeteoDatos.add(new IntervaloMeteoDatosDTO(1, "10Minutal", "", ""));
        intervalosMeteoDatos.add(new IntervaloMeteoDatosDTO(2, "15Minutal", "", ""));
        model.addAttribute("intervalosmeteodatos", intervalosMeteoDatos);

        // TODO Crear e usar intervalosService para consultar os intervalos da base de datos Plataformas
        List<IntervaloPlataformasDTO> intervalosPlataformas = new ArrayList<>();
        intervalosPlataformas.add(new IntervaloPlataformasDTO(1, "DiaADia"));
        intervalosPlataformas.add(new IntervaloPlataformasDTO(2, "MesAMes"));
        model.addAttribute("intervalosplataformas", intervalosPlataformas);

        return "timerestrictionsnew";
    }

    @RequestMapping(value = "admin/timerestrictionspanel/new/create", method = RequestMethod.POST)
    public String createNewTimeRestriction(
            @RequestParam(value = "bbdd", required = true) String bbdd,
            @RequestParam(value = "intervalom", required = true) Integer intervaloMeteoDatos,
            @RequestParam(value = "intervalop", required = true) Integer intervaloPlataformas,
            @RequestParam(value = "error", required = true) String error,
            @RequestParam(value = "tolerancia", required = true) Integer tolerancia,
            @RequestParam(value = "tipo", required = true) Integer tipo
    ) {

        LOGGER.info("\n\n\n----------------------------- INICIO ------------------------------\n\n\n");

        LOGGER.info("bbdd: " + bbdd);
        LOGGER.info("intervalom: " + intervaloMeteoDatos);
        LOGGER.info("intervalop: " + intervaloPlataformas);
        LOGGER.info("error: " + error);
        LOGGER.info("tolerancia: " + tolerancia);
        LOGGER.info("tipo: " + tipo);

        if (bbdd.equals(Constants.RestriccionesCodes.BBDD_METEODATOS_CODE)) {
            LOGGER.info("Base de datos: " + Constants.RestriccionesCodes.BBDD_METEODATOS_CODE);
            restriccionService.gardarRestriccion(bbdd, intervaloMeteoDatos, error, tolerancia, tipo);
        } else if (bbdd.equals(Constants.RestriccionesCodes.BBDD_PLATAFORMAS_CODE)) {
            LOGGER.info("Base de datos: " + Constants.RestriccionesCodes.BBDD_PLATAFORMAS_CODE);
            restriccionService.gardarRestriccion(bbdd, intervaloPlataformas, error, tolerancia, tipo);
        }

        LOGGER.info("\n\n\n----------------------------- FIN ------------------------------\n\n\n");

        return "redirect:/admin/timerestrictionspanel";
    }
}
